﻿// <copyright file="CustomErrorModule.cs" company="N/A (Atif Aziz)">Licensed under the Apache License 2.0. Copyright assigned to Atif Aziz as owner of the ELMAH Google code project.</copyright>
namespace CustomErrorModuleDemo
{
    using System.Collections;
    using System.Web;
    using Elmah;

    /*

        web.config:

            <configuration>
                <system.web>
                    <httpModules>
                        <add name="ErrorLog" type="CustomErrorModuleDemo.CustomErrorModule, CustomErrorModule"/>
                    </httpModules>
                </system.web>
                <-- IIS 7 -->
                <system.webServer>
                    <modules>
                        <add name="ErrorLog" type="CustomErrorModuleDemo.CustomErrorModule, CustomErrorModule" preCondition="managedHandler"/>
                    </modules>
                </system.webServer>
            </configuration>

     */

    /// <summary>
    /// ELMAH Error Signalling - Custom Error Code
    /// https://groups.google.com/group/elmah/browse_thread/thread/7257eeef653de8d1
    /// "Subclass ErrorLogModule and override its GetErrorLog to return [a custom] implementation instead." - Atif Aziz
    /// </summary>
    public class CustomErrorModule : ErrorLogModule
    {
        /// <summary>
        /// Initializes a new instance of the CustomErrorModule class.
        /// </summary>
        /// <remarks>"call the base class implementation to receive the downstream ErrorLog [used] to initialize [the customized] version with." - Atif Aziz (see link to discussion above)</remarks>
        /// <param name="context">HttpContext used to initialize the error log.</param>
        /// <returns>An ELMAH ErrorLog initialized with the given HttpContext.</returns>
        protected override ErrorLog GetErrorLog(HttpContext context)
        {
            var baseErrorLog = base.GetErrorLog(context);
            return new ErrorLogFilter(baseErrorLog);
        }

        /// <summary>
        /// "Create an ErrorLog implementation that is initialized with another." - Atif Aziz (see link to discussion above)
        /// </summary>
        private class ErrorLogFilter : ErrorLog
        {
#if DEBUG
            /// <summary>
            /// Number of times the demo code has stopped to show where to modify the ELMAH error.
            /// </summary>
            //// UnresolvedMergeConflict / important TODO: Remove ErrorLogFilter.numberOfAnnoyingAutomaticDebugBreaks demo code.
            private static int numberOfAnnoyingAutomaticDebugBreaks;
#endif

            /// <summary>
            /// The ErrorLog implementation actually doing the work of logging the errors.
            /// </summary>
            private ErrorLog baseErrorLog;

            /// <summary>
            /// Initializes a new instance of the ErrorLogFilter class.
            /// </summary>
            /// <param name="baseErrorLog">The ErrorLog implementation actually doing the work of logging the errors.</param>
            public ErrorLogFilter(ErrorLog baseErrorLog)
            {
                this.baseErrorLog = baseErrorLog;
            }

            /// <summary>
            /// "Delegate to the downstream in all members..." - Atif Aziz (see link to discussion above)
            /// </summary>
            /// <param name="id">Id of error to return.</param>
            /// <returns>Error with the requested id, if found.</returns>
            public override ErrorLogEntry GetError(string id)
            {
                return this.baseErrorLog.GetError(id);
            }

            /// <summary>
            /// "Delegate to the downstream in all members..." - Atif Aziz (see link to discussion above)
            /// </summary>
            /// <param name="pageIndex">The page containing the errors to return.</param>
            /// <param name="pageSize">The size of the page of errors to return.</param>
            /// <param name="errorEntryList">The list to fill with errors returned?</param>
            /// <returns>The number of errors returned?</returns>
            //// TODO: Properly document all ErrorLogFilter.GetErrors parameters.
            public override int GetErrors(int pageIndex, int pageSize, IList errorEntryList)
            {
                return this.baseErrorLog.GetErrors(pageIndex, pageSize, errorEntryList);
            }

            /// <summary>
            /// "Delegate to the downstream in all members except in Log you can customize the Error object as you like before passing it on." - Atif Aziz (see link to discussion above)
            /// </summary>
            /// <param name="error">ELMAH error to log.</param>
            /// <returns>Id of the error that was logged.</returns>
            public override string Log(Error error)
            {
#if DEBUG
                // UnresolvedMergeConflict / important TODO: Remove ErrorLogFilter.Log demo code.
                if (++ErrorLogFilter.numberOfAnnoyingAutomaticDebugBreaks == 1
                    && System.Diagnostics.Debugger.IsAttached)
                {
                    System.Diagnostics.Debug.WriteLine(error);
                    System.Diagnostics.Debugger.Break();
                }

                /*
                 *   ******************************************************
                 *   *                                                    *
                 *   *    Here is where you can tweak the Elmah error!    *
                 *   *                                                    *
                 *   ******************************************************
                 */
#endif

                // TODO: Adjust Elmah.Error error's properties here.
                return this.baseErrorLog.Log(error);
            }
        }
    }
}
