﻿// <copyright file="CustomSoapExtension.cs" company="N/A (Atif Aziz)">Licensed under the Apache License 2.0. Copyright assigned to Atif Aziz as owner of the ELMAH Google code project.</copyright>
namespace CustomErrorModuleDemo
{
    using System;
    using System.Web;
    using System.Web.Services.Protocols;

    /*

        web.config:

            <configuration>
                <system.web>
                    <webServices>
                        <soapExtensionTypes>
                            <add type="CustomErrorModuleDemo.CustomSoapExtension, CustomErrorModule" group="Low" priority="1"/>
                        </soapExtensionTypes>
                    </webServices>
                </system.web>
            </configuration>

     */

    /// <summary>
    /// <para>
    /// Handle exceptions in web services with Elmah
    /// http://stackoverflow.com/questions/2180228/handle-exceptions-in-web-services-with-elmah
    /// "You can use a SoapExtension to do this" - Liam Corner
    /// </para>
    /// <para>
    /// Using SOAP Extensions in ASP.NET
    /// http://msdn.microsoft.com/en-us/magazine/cc164007.aspx
    /// "SoapExtensions only work with SOAP requests" - George Shepherd
    /// (Debugging with .ASMX test page will not trigger ProcessMessage)
    /// </para>
    /// </summary>
    public class CustomSoapExtension : SoapExtension
    {
        /// <summary>
        /// Allows a SOAP extension to receive a System.Web.Services.Protocols.SoapMessage to process at each System.Web.Services.Protocols.SoapMessageStage.
        /// </summary>
        /// <param name="message">The System.Web.Services.Protocols.SoapMessage to process.</param>
        public override void ProcessMessage(SoapMessage message)
        {
            if (message != null
                && message.Stage == SoapMessageStage.AfterSerialize
                && message.Exception != null
                && HttpContext.Current != null)
            {
                Elmah.ErrorSignal.FromCurrentContext().Raise(message.Exception.InnerException);
            }
        }

        /// <summary>
        /// Allows a SOAP extension to initialize data specific to a class implementing an XML Web service at a one time performance cost.
        /// </summary>
        /// <param name="serviceType">The type of the class implementing the XML Web service to which the SOAP extension is applied.</param>
        /// <returns>The System.Object that the SOAP extension initializes for caching.</returns>
        public override object GetInitializer(System.Type serviceType)
        {
            return null;
        }

        /// <summary>
        /// Allows a SOAP extension to initialize data specific to an XML Web service method using an attribute applied to the XML Web service method at a one time performance cost.
        /// </summary>
        /// <param name="methodInfo">A System.Web.Services.Protocols.LogicalMethodInfo representing the specific function prototype for the XML Web service method to which the SOAP extension is applied.</param>
        /// <param name="attribute">The System.Web.Services.Protocols.SoapExtensionAttribute applied to the XML Web service method.</param>
        /// <returns>The System.Object that the SOAP extension initializes for caching.</returns>
        public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute)
        {
            return null;
        }

        /// <summary>
        /// Allows a SOAP extension to initialize itself using the data cached in the System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute) method.
        /// </summary>
        /// <param name="initializer">The System.Object returned from System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute) cached by ASP.NET.</param>
        public override void Initialize(object initializer)
        {
        }
    }
}