#region License, Terms and Author(s)
//
// ELMAH - Error Logging Modules and Handlers for ASP.NET
// Copyright (c) 2004-9 Atif Aziz. All rights reserved.
//
//  Author(s):
//
//      Atif Aziz, http://www.raboof.com
//      Lee Kelleher, http://leekelleher.com
//      Rajiv Kalra
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
#endregion

// All code in this file requires .NET Framework 2.0 or later.

#if !NET_1_1 && !NET_1_0

namespace Elmah
{
	#region Imports

	using System;
	using System.Configuration;
	using System.Data;
	using System.Globalization;
	using System.Xml;

	using Elmah;
	using MySql.Data.MySqlClient;

	using IDictionary = System.Collections.IDictionary;
	using IList = System.Collections.IList;

	#endregion

	/// <summary>
	/// An <see cref="ErrorLog"/> implementation that uses MySql as its backing store.
	/// </summary>
	public class MySqlErrorLog : ErrorLog
	{
		private readonly string _connectionString;

		private const int _maxAppNameLength = 60;

		/// <summary>
		/// Initializes a new instance of the <see cref="MySqlErrorLog"/> class using a dictionary of configured settings.
		/// </summary>
		public MySqlErrorLog(IDictionary config)
		{
			if (config == null)
				throw new ArgumentNullException("config");

			ConnectionStringSettings settings = ConfigurationManager.ConnectionStrings["ELMAH_MySql"];

			if (settings == null)
				throw new ConfigurationErrorsException("Connection string is missing for the MySQL error log.");

			string connectionString = settings.ConnectionString;

			// If there is no connection string to use then throw an exception to abort construction.
			if (connectionString.Length == 0)
				throw new ApplicationException("Connection string is missing for the MySql error log.");

			_connectionString = connectionString;

			// Set the application name as this implementation provides per-application isolation over a single store.
			string appName = (string)config["applicationName"] ?? string.Empty;

			if (appName.Length > _maxAppNameLength)
			{
				throw new ApplicationException(string.Format(
					"Application name is too long. Maximum length allowed is {0} characters.",
					_maxAppNameLength.ToString("N0")));
			}

			ApplicationName = appName;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="MySqlErrorLog"/> class to use a specific connection string for connecting to the database.
		/// </summary>
		public MySqlErrorLog(string connectionString)
		{
			if (connectionString == null)
				throw new ArgumentNullException("connectionString");

			if (connectionString.Length == 0)
				throw new ArgumentException(null, "connectionString");

			_connectionString = connectionString;
		}

		/// <summary>
		/// Gets the name of this error log implementation.
		/// </summary>
		public override string Name
		{
			get { return "MySql Error Log"; }
		}

		/// <summary>
		/// Gets the connection string used by the log to connect to the database.
		/// </summary>
		public virtual string ConnectionString
		{
			get { return _connectionString; }
		}

		/// <summary>
		/// Logs an error to the database.
		/// </summary>
		public override string Log(Error error)
		{
			if (error == null)
				throw new ArgumentNullException("error");

			string errorXml = ErrorXml.EncodeString(error);

			const string query = @"
				INSERT INTO ELMAH_Error (
					Application, 
					Host, 
					Type, 
					Source, 
					Message, 
					User, 
					StatusCode, 
					TimeUtc, 
					AllXml
				)
                VALUES (
					?Application, 
					?Host, 
					?Type, 
					?Source, 
					?Message, 
					?User, 
					?StatusCode, 
					?TimeUtc, 
					?AllXml
				);";

			using (MySqlConnection connection = new MySqlConnection(ConnectionString))
			{
				using (MySqlCommand command = new MySqlCommand(query, connection))
				{
					MySqlParameterCollection parameters = command.Parameters;

					parameters.Add("?Application", MySqlDbType.String, 60).Value = this.ApplicationName;
					parameters.Add("?Host", MySqlDbType.String, 30).Value = error.HostName;
					parameters.Add("?Type", MySqlDbType.String, 100).Value = error.Type;
					parameters.Add("?Source", MySqlDbType.String, 60).Value = error.Source;
					parameters.Add("?Message", MySqlDbType.String, 500).Value = error.Message;
					parameters.Add("?User", MySqlDbType.String, 50).Value = error.User;
					parameters.Add("?StatusCode", MySqlDbType.Int64).Value = error.StatusCode;
					parameters.Add("?TimeUtc", MySqlDbType.Datetime).Value = error.Time.ToUniversalTime();
					parameters.Add("?AllXml", MySqlDbType.String).Value = errorXml;

					connection.Open();

					return command.ExecuteNonQuery().ToString();
				}
			}
		}

		/// <summary>
		/// Returns a page of errors from the databse in descending order of logged time.
		/// </summary>
		public override int GetErrors(int pageIndex, int pageSize, IList errorEntryList)
		{
			if (pageIndex < 0)
				throw new ArgumentOutOfRangeException("pageIndex", pageIndex, null);

			if (pageSize < 0)
				throw new ArgumentOutOfRangeException("pageSize", pageSize, null);

			const string sql = @"
				SELECT
					ErrorId,
					Application,
					Host,
					Type,
					Source,
					Message,
					User,
					StatusCode,
					TimeUtc
				FROM
					ELMAH_Error
				ORDER BY
					ErrorId DESC
				LIMIT 
					?RowStart,
					?RowEnd;
				
				SELECT COUNT(*) FROM ELMAH_Error;";

			using (MySqlConnection connection = new MySqlConnection(ConnectionString))
			{
				using (MySqlCommand command = new MySqlCommand(sql, connection))
				{
					MySqlParameterCollection parameters = command.Parameters;

					parameters.Add("?RowStart", MySqlDbType.Int16).Value = (pageIndex * pageSize);
					parameters.Add("?RowEnd", MySqlDbType.Int16).Value = pageSize;

					connection.Open();

					using (MySqlDataReader reader = command.ExecuteReader())
					{
						if (errorEntryList != null)
						{
							while (reader.Read())
							{
								string id = reader["ErrorId"].ToString();

								Error error = new Error();

								error.ApplicationName = reader["Application"].ToString();
								error.HostName = reader["Host"].ToString();
								error.Type = reader["Type"].ToString();
								error.Source = reader["Source"].ToString();
								error.Message = reader["Message"].ToString();
								error.User = reader["User"].ToString();
								error.StatusCode = Convert.ToInt32(reader["StatusCode"]);
								error.Time = Convert.ToDateTime(reader["TimeUtc"]).ToLocalTime();

								errorEntryList.Add(new ErrorLogEntry(this, id, error));
							}
						}

						// Get the result of SELECT COUNT(*) FROM Page
						reader.NextResult();
						reader.Read();
						return reader.GetInt32(0);
					}
				}
			}
		}

		/// <summary>
		/// Returns the specified error from the database, or null if it does not exist.
		/// </summary>
		public override ErrorLogEntry GetError(string id)
		{
			if (id == null)
				throw new ArgumentNullException("id");

			if (id.Length == 0)
				throw new ArgumentException(null, "id");

			long key;

			try
			{
				key = long.Parse(id, CultureInfo.InvariantCulture);
			}
			catch (FormatException e)
			{
				throw new ArgumentException(e.Message, "id", e);
			}

			const string sql = @"
                SELECT 
                    AllXml
                FROM 
                    ELMAH_Error
                WHERE
                    ErrorId = ?ErrorId";

			using (MySqlConnection connection = new MySqlConnection(ConnectionString))
			{
				using (MySqlCommand command = new MySqlCommand(sql, connection))
				{
					MySqlParameterCollection parameters = command.Parameters;
					parameters.Add("?ErrorId", MySqlDbType.Int64).Value = key;

					connection.Open();

					string errorXml = (string)command.ExecuteScalar();

					if (errorXml == null)
						return null;

					Error error = ErrorXml.DecodeString(errorXml);
					return new ErrorLogEntry(this, id, error);
				}
			}
		}
	}
}

#endif //!NET_1_1 && !NET_1_0
