//
//  Author(s):
//
//		Scott Wilson (sw@scratchstudio.net)
//	
//	A contribution to ELMAH originally written by:
//
//      Atif Aziz (atif.aziz@skybow.com, http://www.raboof.com)
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using System.Web;
using System.IO;
using System.Xml;
using System.Collections;
using System.Configuration;
using MySql.Data.MySqlClient;
using GotDotNet.Elmah;

namespace GotDotNet.Elmah
{
	/// <summary>
	/// An <see cref="ErrorLog"/> implementation that uses MySql  
	/// as its backing store.
	/// </summary>
	public class MySqlErrorLog : GotDotNet.Elmah.ErrorLog
	{
		private string connectionString;

		/// <summary>
		/// Initializes a new instance of the <see cref="MySqlErrorLog"/> class
		/// using a dictionary of configured settings.
		/// </summary>		
		public MySqlErrorLog(IDictionary config)
		{
			if (config["connectionString"] != null)
			{
				connectionString = (string)config["connectionString"];
			}
			else
			{
				throw new System.ApplicationException("Log Path is missing for the Xml error log.");
			}	
		}
		
		/// <summary>
		/// Initializes a new instance of the <see cref="MySqlErrorLog"/> class
		/// using a specific connection string.
		/// </summary>
		public MySqlErrorLog(string connectionString)
		{
				this.connectionString = connectionString;
		}
		
		/// <summary>
		/// Gets the name of this error log implementation.
		/// </summary>
		public override string Name
		{
			get { return "MySql Error Log"; }
		}

		/// <summary>
		/// Logs an error to the database.
		/// </summary>
		public override void Log(GotDotNet.Elmah.Error error)
		{
			Guid errorId = Guid.NewGuid();

            StringWriter errorStringWriter = new StringWriter();
            XmlTextWriter errorXmlWriter = new XmlTextWriter(errorStringWriter);
            errorXmlWriter.Formatting = Formatting.Indented;

            errorXmlWriter.WriteStartElement("error");
            error.ToXml(errorXmlWriter);
            errorXmlWriter.WriteEndElement();
            errorXmlWriter.Flush();
            
            string errorXml = errorStringWriter.ToString();			
			
			string sqlStatement = @"

						INSERT INTO Elmah.Error (ErrorId, Application, Host, Type, Source, Message, User, AllXml, StatusCode, TimeUtc)
						VALUES (?id, ?appname, ?hostname, ?type, ?source, ?message, ?user, ?xml, ?status, ?time)
						
						";  

            using (MySqlConnection c = new MySqlConnection(connectionString))
            {
            	c.Open();
	            MySqlCommand cmd = new MySqlCommand(sqlStatement, c);
	            cmd.Parameters.Add(new MySqlParameter("?id", errorId.ToString()));
	            cmd.Parameters.Add(new MySqlParameter("?appname", error.ApplicationName));
	            cmd.Parameters.Add(new MySqlParameter("?hostname", error.HostName));
	            cmd.Parameters.Add(new MySqlParameter("?type", error.Type));
	            cmd.Parameters.Add(new MySqlParameter("?source", error.Source));
	            cmd.Parameters.Add(new MySqlParameter("?message", error.Message));
	            cmd.Parameters.Add(new MySqlParameter("?user", error.User));
	            cmd.Parameters.Add(new MySqlParameter("?xml", errorXml));
	            cmd.Parameters.Add(new MySqlParameter("?status", error.StatusCode));
	            cmd.Parameters.Add(new MySqlParameter("?time", error.Time));
	            cmd.ExecuteNonQuery();
				cmd.Dispose();
            }
		}

		/// <summary>
		/// Returns a page of errors from the databse in descending order 
		/// of logged time.
		/// </summary>
		public override int GetErrors(int pageIndex, int pageSize, IList errorEntryList)
		{
			if (pageIndex < 0)
			{
				throw new ArgumentOutOfRangeException("pageIndex");
			}

			if (pageSize < 0)
			{
				throw new ArgumentOutOfRangeException("pageSize");
			}
			
			/* Find out how many errors there are */
			string sqlStatementCount = @"SELECT COUNT(*) AS count FROM Elmah.Error";
						
			/* Get the relevant errors from the DB and rehydrate them */
			string sqlStatement = @"
				
				SELECT ErrorID, AllXml FROM Elmah.Error
				ORDER BY TimeUtc DESC
				LIMIT ?pageSize OFFSET ?pageOffset
				
			";
			
			int count;
			
            using (MySqlConnection c = new MySqlConnection(connectionString))
            {
            	c.Open();
	            MySqlCommand countCmd = new MySqlCommand(sqlStatementCount, c);
	            count = Convert.ToInt32(countCmd.ExecuteScalar());
				countCmd.Dispose();
				
				MySqlCommand cmd = new MySqlCommand(sqlStatement, c);
				cmd.Parameters.Add(new MySqlParameter("?pageSize", pageSize));
				cmd.Parameters.Add(new MySqlParameter("?pageOffset", pageSize * pageIndex));
	            MySqlDataReader r = cmd.ExecuteReader();
	            
	            while (r.Read())
	            {	
	            	StringReader sr = new StringReader(r.GetString(1));	            	
	            	XmlTextReader reader = new XmlTextReader(sr);
					GotDotNet.Elmah.Error error = new GotDotNet.Elmah.Error();
					error.FromXml(reader);
					
					string id = r.GetString(0);
					
					if (errorEntryList != null)
					{
						errorEntryList.Add(new GotDotNet.Elmah.ErrorLogEntry(this, id, error));
					}	
		        }
				cmd.Dispose();
            }

			return count;

		}

		/// <summary>
		/// Returns the specified error from the database, or null 
		/// if it does not exist.
		/// </summary>
		public override GotDotNet.Elmah.ErrorLogEntry GetError(string id)
		{
			/* Get the error from the DB and rehydrate it */
			string sqlStatement = @"
				
				SELECT AllXml FROM Elmah.Error WHERE ErrorId = ?errorId
				
			";			
			
			string errorString;
			
            using (MySqlConnection c = new MySqlConnection(connectionString))
            {
            	c.Open();
	            MySqlCommand cmd = new MySqlCommand(sqlStatement, c);
	            MySqlParameter param = new MySqlParameter("?errorId", id);
	            cmd.Parameters.Add(param);
	            errorString = (string)cmd.ExecuteScalar();
				cmd.Dispose();
            }
            
			if ((errorString == null) || (errorString.Length == 0))
			{
				return null;
			}

            StringReader sr = new StringReader(errorString);
			XmlTextReader reader = new XmlTextReader(sr);
			GotDotNet.Elmah.Error error = new GotDotNet.Elmah.Error();
			error.FromXml(reader);
			reader.Close();
			return new GotDotNet.Elmah.ErrorLogEntry(this, id, error);
		}
	}
}
